@@ -272,7 +272,8 @@ return them with the flag."
 					(type :case-ignore) (checked :bool))
 	      (when name
 		(cond
-		 ((string= type "submit")
+		 ((or (string= type "submit")
+		      (string= type "image"))
 		  ;; Submit button input, not set name and value here.
 		  ;; They are set in `w3m-form-submit'.
 		  nil)
@@ -355,7 +356,8 @@ return them with the flag."
 		type (match-string 2 fid)
 		name (match-string 3 fid))
 	  (cond
-	   ((string= type "submit")
+	   ((or (string= type "submit")
+		(string= type "image"))
 	    ;; Remove status to support forms containing multiple
 	    ;; submit buttons.
 	    (w3m-form-put form name nil))
@@ -441,7 +443,8 @@ return them with the flag."
 		 w3m-form-face
 		 w3m-action (w3m-form-input-map ,form ,name)
 		 w3m-cursor-anchor (w3m-form-input-map ,form ,name))))
-	     ((string= type "submit")
+	     ((or (string= type "submit")
+		  (string= type "image"))
 	      (w3m-form-make-button
 	       start (point)
 	       `(w3m-action (w3m-form-submit ,form ,name ,value)
@@ -532,31 +535,32 @@ return them with the flag."
 
 
 (defun w3m-form-replace (string &optional invisible)
-  (save-excursion
-    (let* ((start (text-property-any
-		   (point-min)
-		   (point-max)
-		   'w3m-action
-		   (w3m-get-text-property-around 'w3m-action)))
-	   (width (string-width
-		   (buffer-substring
-		    start
-		    (next-single-property-change start 'w3m-action))))
-	   (prop (text-properties-at start))
-	   (buffer-read-only))
-      (goto-char start)
-      (insert (setq string
-		    (if invisible
-			(make-string (length string) ?.)
-		      (mapconcat 'identity
-				 (split-string
-				  (truncate-string string width) "\n")
-				 "")))
-	      (make-string (- width (string-width string)) ?\ ))
-      (delete-region (point)
-		     (next-single-property-change (point) 'w3m-action))
-      (add-text-properties start (point) prop)
-      (point))))
+  (let* ((start (text-property-any
+		 (point-min)
+		 (point-max)
+		 'w3m-action
+		 (w3m-get-text-property-around 'w3m-action)))
+	 (width (string-width
+		 (buffer-substring
+		  start
+		  (next-single-property-change start 'w3m-action))))
+	 (prop (text-properties-at start))
+	 (p (point))
+	 (buffer-read-only))
+    (goto-char start)
+    (insert (setq string
+		  (if invisible
+		      (make-string (length string) ?.)
+		    (mapconcat 'identity
+			       (split-string
+				(truncate-string string width) "\n")
+			       "")))
+	    (make-string (- width (string-width string)) ?\ ))
+    (delete-region (point)
+		   (next-single-property-change (point) 'w3m-action))
+    (add-text-properties start (point) prop)
+    (prog1 (point)
+      (goto-char p))))
 
 (defun w3m-form-input (form name type width maxlength value)
   (save-excursion
@@ -631,26 +635,27 @@ character."
     (nreverse (cons (substring text start) parts))))
 
 (defun w3m-form-textarea-replace (hseq string)
-  (save-excursion
-    (let ((s (get-text-property (point) 'w3m-form-hseq))
-	  (hseq (abs hseq))
-	  (chopped (w3m-form-text-chop string))
-	  cs next)
-      (unless (and s (eq s hseq))
-	(goto-char (point-min))
-	(while (and (not (eobp))
-		    (not (eq hseq (get-text-property (point) 'w3m-form-hseq))))
-	  (goto-char (next-single-property-change (point)
-						  'w3m-form-hseq))))
-      (while chopped
-	(w3m-form-replace (car chopped))
-	(goto-char (next-single-property-change (point) 'w3m-form-hseq)) ; end
-	(when (setq next (next-single-property-change (point) 'w3m-form-hseq))
-	  (goto-char next))
-	(setq cs (get-text-property (point) 'w3m-form-hseq))
-	(setq chopped
-	      (if (and cs (eq (abs cs) hseq))
-		  (cdr chopped)))))))
+  (let ((s (get-text-property (point) 'w3m-form-hseq))
+	(hseq (abs hseq))
+	(chopped (w3m-form-text-chop string))
+	(p (point))
+	cs next)
+    (unless (and s (eq s hseq))
+      (goto-char (point-min))
+      (while (and (not (eobp))
+		  (not (eq hseq (get-text-property (point) 'w3m-form-hseq))))
+	(goto-char (next-single-property-change (point)
+						'w3m-form-hseq))))
+    (while chopped
+      (w3m-form-replace (car chopped))
+      (goto-char (next-single-property-change (point) 'w3m-form-hseq)) ; end
+      (when (setq next (next-single-property-change (point) 'w3m-form-hseq))
+	(goto-char next))
+      (setq cs (get-text-property (point) 'w3m-form-hseq))
+      (setq chopped
+	    (if (and cs (eq (abs cs) hseq))
+		(cdr chopped))))
+    (goto-char p)))
 
 (defun w3m-form-textarea-info ()
   "Return a cons cell of (NAME . LINE) for current text area."
@@ -670,7 +675,7 @@ character."
 				 'w3m-form-hseq))
 		     (goto-char next)))
 	  (incf lines)))
-      (cons (get-text-property (point) 'w3m-form-name) lines))))
+      (cons (w3m-get-text-property-around 'w3m-form-name) lines))))
 
 (defvar w3m-form-input-textarea-keymap nil)
 (unless w3m-form-input-textarea-keymap
